program crlf;
{$APPTYPE CONSOLE}
uses
  Classes,
  SysUtils,
  Windows, 
  PCharUtils in 'PCharUtils.pas',
  crlfLexer in 'crlfLexer.pas',
  crlfParser in 'crlfParser.pas',
  FindFileIter in 'FindFileIter.pas';

const
  ProcessedExtension = '.pi';
  RenameExtension = '.bak';

procedure Syntax;
begin
  Writeln(
    'CRLF to CR and vice versa line-break converter v1.0'#10,
    '  Copyright (C) 2002 Alexey Barkovoy (clootie@reactor.ru)'#10,
    'Syntax:  ' + ExtractFileName(ParamStr(0)) + ' [-options] <input files>...'#10,
    'Options:'#10,
    ' h, ?    This help'#10,
    ' CRLF    Convert line breaks to CRLF'#10,
    ' LF      Convert line breaks to LF'#10,
    ' Auto    Auto-determine output format (CRLF or LF)'#10,
    '         - switch used line breaks to another'#10,
    ' r       Rename old files to ',
        RenameExtension, ' and write new files to original filename'#10,
    ' t       Save timestamps of original files'#10,
    #10,
    'Options "-Auto -t" are assumed if nothing is selected.'#10,
    'Output files will be written to new files with extension ',
        ProcessedExtension
  );
  Halt(2);
end;

////////////////////////////////////////////////////////////////////////////
var
  OptionsRename: Boolean;       // Rename files or create new?
  OptionsBreaks: TcrlfFlow;     // What to convert?
  OptionsTStamp: Boolean;       // Save old time-stamps?
////////////////////////////////////////////////////////////////////////////


procedure Process(const AOld, ANew: string);
var
  parse: TcrlfParser;
  fsIn, fsOut: TStream;
  answer: String;

  CreationTime: TFileTime;
  LastAccessTime: TFileTime;
  LastWriteTime: TFileTime;
begin
  fsOut := nil;
  parse := nil;
  fsIn := nil;
  try
    fsIn := TFileStream.Create(AOld, fmOpenRead);
    parse := TcrlfParser.Create(fsIn, OptionsBreaks);
    answer := parse.Parse;

    try // Output results to file
      fsOut := TFileStream.Create(ANew, fmCreate);
      fsOut.WriteBuffer(Pointer(answer)^, Length(answer));
    finally
      if OptionsTStamp then // Copy file TimeStamps
      begin
        GetFileTime((fsIn as TFileStream).Handle, @CreationTime, @LastAccessTime, @LastWriteTime);
        SetFileTime((fsOut as TFileStream).Handle, @CreationTime, @LastAccessTime, @LastWriteTime);
      end;
      fsOut.Free;
    end;
  finally
    parse.Free;
    fsIn.Free;
  end;
end;

procedure Params(ACommandLine: PChar);
var
  processed: TStringList; // avoid processing files twice

  function HandleOptions(cp: PChar): PChar;

  var
    tmp: string;
  begin
    OptionsRename:= False;
    OptionsBreaks:= flAuto;
    OptionsTStamp:= True;

    cp := SkipWhite(cp);

    while cp^ = '-' do
    begin
      Inc(cp);

      case cp^ of
        'h', 'H', '?':
          Syntax;

        'r', 'R':
        begin
          Inc(cp);
          OptionsRename:= True;
        end;

        'T', 't':
        begin
          Inc(cp);
          OptionsTStamp := True;
        end;

        'C', 'c', 'L', 'l', 'A', 'a':
        begin
          cp := ReadIdent(cp, tmp);
          tmp := UpperCase(tmp);
          if tmp = 'CRLF' then OptionsBreaks:= flLFtoCRLF
          else if tmp = 'LF' then OptionsBreaks:= flCRLFtoLF
          else if tmp = 'AUTO' then OptionsBreaks:= flAuto;
        end;
      else
        Syntax;
      end;

      cp := SkipWhite(cp);
    end;
    Result := cp;
  end;

  function HandleFiles(cp: PChar): PChar;
  var
    tmp: string;
    iter: IFindFileIterator;
    ind: Integer;
  begin
    while not (cp^ in ['-', #0]) do
    begin
      cp := SkipWhite(ReadStringDoubleQuotedMaybe(cp, tmp));
      if CreateFindFile(tmp, faAnyFile and not faDirectory, iter) then
        repeat
          try
            if not processed.Find(iter.Name, ind) then
            begin
              processed.Add(iter.Name);
              tmp := ExtractFileDir(iter.Name);
              if OptionsRename then
              begin
                tmp := ChangeFileExt(iter.Name, RenameExtension);
                if not RenameFile(iter.Name, tmp) then
                  raise Exception.CreateFmt('Couldn''t rename %s to %s',
                    [iter.Name, tmp]);
                Process(tmp, iter.Name);
              end
              else
                Process(iter.Name, ChangeFileExt(iter.Name, ProcessedExtension));
            end;
          except
            on e: Exception do
              Writeln('Error: ', iter.Name, ': ', e.Message);
          end;
        until not iter.Next
      else
        Writeln('Could not find ', tmp);
    end;
    Result := cp;
  end;

var
  cp: PChar;
  tmp: String;
begin
  cp := ACommandLine;

  processed := nil;
  try
    processed := TStringList.Create;
    repeat
      cp := SkipWhite(ReadStringDoubleQuotedMaybe(cp, tmp));
      cp := HandleOptions(cp);
      cp := HandleFiles(cp);
    until cp^ = #0;
  finally
    processed.Free;
  end;
end;

begin
  try
    if ParamCount = 0 then Syntax
    else Params(CmdLine);
  except
    on e: Exception do
    begin
      Writeln(e.Message);
      Halt(1);
    end;
  end;
end.

